<?php
if(!defined('ABSPATH')) exit;

new Cdash_Events_Metaboxes($cdash_events);

class Cdash_Events_Metaboxes
{
	private $options = array();
	private $metaboxes = array();
	private $recurrences = array();
	private $tickets_fields = array();
	private $cdash_events;


	public function __construct($cdash_events)
	{
		$this->events_maker = $cdash_events;

		//settings
		$this->options = $cdash_events->get_options();

		//actions
		add_action('add_meta_boxes', array(&$this, 'add_events_meta_boxes'), 10, 2);
		add_action('admin_enqueue_scripts', array(&$this, 'admin_scripts_styles'));
		add_action('after_setup_theme', array(&$this, 'set_recurrences'));
		add_action('after_setup_theme', array(&$this, 'load_defaults'));
		add_action('save_post', array(&$this, 'save_event'), 10, 2);
		add_action( 'plugins_loaded', array(&$this, 'completely_delete') );
		add_filter( 'page_row_actions', array(&$this, 'event_row_actions'), 10, 2 );
		add_action( 'wp_ajax_cde_apply_changes_to_series', array(&$this, 'apply_changes_to_series') );
	}


	/**
	 * 
	*/
	public function set_recurrences()
	{
		$this->recurrences = $this->events_maker->get_recurrences();
	}


	/**
	 * 
	*/
	public function load_defaults()
	{
		$this->tickets_fields = apply_filters(
			'cde_event_tickets_fields',
			array(
				'name' => __('Ticket Name', 'cdash-events'),
				'price' => __('Price', 'cdash-events')
			)
		);

		$post_types = apply_filters('cde_event_post_type', array('event'));

		foreach ($post_types as $post_type)
		{
			$this->metaboxes[] = apply_filters(
				'cde_'.$post_type.'_metaboxes',
				array(
					'event-options-box' => array(
						'title' => __('Event Display Options', 'cdash-events'),
						'callback' => array(&$this, 'event_options_cb'),
						'post_type' => $post_type,
						'context' => 'side',
						'priority' => 'core'
					),
					'event-date-time-box' => array(
						'title' => __('Event Date and Time', 'cdash-events'),
						'callback' => array(&$this, 'event_date_time_cb'),
						'post_type' => $post_type,
						'context' => 'normal',
						'priority' => 'high'
					),
					'event-cost-tickets-box' => array(
						'title' => __('Event Tickets', 'cdash-events'),
						'callback' => array(&$this, 'event_tickets_cb'),
						'post_type' => $post_type,
						'context' => 'normal',
						'priority' => 'high'
					)
				)
			);
		}
	}


	/**
	 * 
	*/
	public function admin_scripts_styles($page)
	{
		$screen = get_current_screen();

		if(($page === 'post-new.php' || $page === 'post.php') && in_array($screen->post_type, apply_filters('cde_event_post_type', array('event'))))
		{
			global $wp_locale;

			wp_register_script(
				'cdash-events-datetimepicker',
				CDASH_EVENTS_URL.'/assets/jquery-timepicker-addon/jquery-ui-timepicker-addon.min.js',
				array('jquery')
			);
			
			$lang = str_replace('_', '-', get_locale());
			$lang_exp = explode('-', $lang);

			if(file_exists(CDASH_EVENTS_PATH.'assets/jquery-timepicker-addon/i18n/jquery-ui-timepicker-'.$lang.'.js'))
				$lang_path = CDASH_EVENTS_URL.'/assets/jquery-timepicker-addon/i18n/jquery-ui-timepicker-'.$lang.'.js';
			elseif(file_exists(CDASH_EVENTS_PATH.'assets/jquery-timepicker-addon/i18n/jquery-ui-timepicker-'.$lang_exp[0].'.js'))
				$lang_path = CDASH_EVENTS_URL.'/assets/jquery-timepicker-addon/i18n/jquery-ui-timepicker-'.$lang_exp[0].'.js';
			
			if(isset($lang_path))
			{
				wp_register_script(
					'cdash-events-datetimepicker-localization',
					$lang_path,
					array('jquery', 'cdash-events-datetimepicker')
				);

				wp_enqueue_script('cdash-events-datetimepicker-localization');
			}

			wp_register_script(
				'cdash-events-admin-post',
				CDASH_EVENTS_URL.'/js/admin-post.js',
				array('jquery', 'jquery-ui-core', 'jquery-ui-datepicker', 'jquery-ui-slider', 'cdash-events-datetimepicker')
			);

			wp_enqueue_script('cdash-events-admin-post');

			wp_localize_script(
				'cdash-events-admin-post',
				'emPostArgs',
				array(
					'ticketsFields' => $this->tickets_fields,
					'ticketDelete' => __('Delete', 'cdash-events'),
					'currencySymbol' => cde_get_currency_symbol(),
					'startDateTime' => __('Start date/time', 'cdash-events'),
					'endDateTime' => __('End date/time', 'cdash-events'),
					'dateDelete' => __('Delete', 'cdash-events'),
					'deleteTicket' => __('Are you sure you want to delete this ticket?', 'cdash-events'),
					'deleteCustomOccurrence' => __('Are you sure you want to delete this occurrence?', 'cdash-events'),
					'firstWeekDay' => $this->options['general']['first_weekday'],
					'monthNames' => array_values($wp_locale->month),
					'monthNamesShort' => array_values($wp_locale->month_abbrev),
					'dayNames' => array_values($wp_locale->weekday),
					'dayNamesShort' => array_values($wp_locale->weekday_abbrev),
					'dayNamesMin' => array_values($wp_locale->weekday_initial),
					'isRTL' => $wp_locale->is_rtl(),
					'day' => __('day', 'cdash-events'),
					'days' => __('days', 'cdash-events'),
					'week' => __('week', 'cdash-events'),
					'weeks' => __('weeks', 'cdash-events'),
					'month' => __('month', 'cdash-events'),
					'months' => __('months', 'cdash-events'),
					'year' => __('year', 'cdash-events'),
					'years' => __('years', 'cdash-events')
				)
			);

			wp_localize_script( 'cdash-events-admin-post', 'eventajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );

			wp_register_style(
				'cdash-events-admin',
				CDASH_EVENTS_URL.'/css/admin.css'
			);

			wp_register_style(
				'cdash-events-datetimepicker',
				CDASH_EVENTS_URL.'/assets/jquery-timepicker-addon/jquery-ui-timepicker-addon.min.css'
			);

			wp_enqueue_style('cdash-events-admin');
			wp_enqueue_style('cdash-events-wplike');
			wp_enqueue_style('cdash-events-datetimepicker');
		}
	}


	/**
	 * Add event metaboxes
	*/
	public function add_events_meta_boxes($post_type, $post)
	{
		if (isset($post_type) && in_array($post_type, apply_filters('cde_event_post_type', array('event'))))
		{
		
			global $wp_meta_boxes;
	
			foreach($this->metaboxes as $key)
			{
				foreach($key as $id => $metabox)
				{
					add_meta_box($id, $metabox['title'], $metabox['callback'], $metabox['post_type'], $metabox['context'], $metabox['priority']);
				}
			}
	
			$found_priority = false;
	
			foreach(array('low', 'core', 'high') as $priority)
			{
				if(isset($wp_meta_boxes[$post->post_type]['side'][$priority]['postimagediv']))
				{
					$found_priority = $priority;
					$post_image_box = $wp_meta_boxes[$post->post_type]['side'][$priority]['postimagediv'];
					break;
				}
			}
	
			$sideboxes = array();
			$event_options_box = $wp_meta_boxes[$post->post_type]['side']['core']['event-options-box'];
	
			unset($wp_meta_boxes[$post->post_type]['side']['core']['event-options-box']);
	
			if($found_priority !== false)
				unset($wp_meta_boxes[$post->post_type]['side'][$found_priority]['postimagediv']);
	
			foreach($wp_meta_boxes[$post->post_type]['side']['core'] as $id => $sidebox)
			{
				$sideboxes[$id] = $sidebox;
	
				if($id === 'submitdiv')
				{
					$sideboxes['event-options-box'] = $event_options_box;
	
					if($found_priority !== false)
						$sideboxes['postimagediv'] = $post_image_box;
				}
			}
	
			$wp_meta_boxes[$post->post_type]['side']['core'] = $sideboxes;
		}
	}


	/**
	 * Event date & time metabox callback
	*/
	public function event_date_time_cb($post)
	{
		wp_nonce_field('cdash_events_save_event_datetime', 'event_nonce_datetime');

		// defaults
		$options = array('weekly' => '', 'monthly' => '');
		$selected = '';

		// datetimes
		$current_date = date('Y-m-d', current_time('timestamp'));
		$current_time = date('H:i', current_time('timestamp'));
		$today = (int)date('N', current_time('timestamp'));

		// metas
		$event_all_day = get_post_meta($post->ID, '_event_all_day', true);
		$event_start_date = explode(' ', get_post_meta($post->ID, '_event_start_date', true));
		$event_end_date = explode(' ', get_post_meta($post->ID, '_event_end_date', true));
		$event_recurrence = get_post_meta($post->ID, '_event_recurrence', true);

		// edit event?
		if($event_all_day !== '')
		{
			$event_all_day = (bool)(int)$event_all_day;

			if($event_all_day)
				$event_start_date[1] = $event_end_date[1] = '';
		}
		else
		{
			$event_all_day = false;

			$event_start_date[0] = $event_end_date[0] = $current_date;
			$event_start_date[1] = $event_end_date[1] = $current_time;
		}

		// creating new event?
		if(!is_array($event_recurrence))
		{
			$event_recurrence = array(
				'type' => 'once',
				'repeat' => 1,
				'until' => $current_date,
				'weekly_days' => array($today),
				'monthly_day_type' => 1,
				'separate_end_date' => array()
			);
		}
		$html = '
		<div class="date-time-row">
			<label for="event_start_date">'.__('Start date/time', 'cdash-events').':</label> <input id="event_start_date" class="format-date" type="text" name="event_start_date" value="'.esc_attr($event_start_date[0]).'"/> <input id="event_start_time" class="format-time" type="text" name="event_start_time" value="'.esc_attr(isset($event_start_date[1]) ? substr($event_start_date[1], 0, 5) : '').'" '.($event_all_day ? 'style="display: none;"' : '').'/>
		</div>
		<div class="date-time-row">
			<label for="event_end_date">'.__('End date/time', 'cdash-events').':</label> <input id="event_end_date" class="format-date" type="text" name="event_end_date" value="'.esc_attr($event_end_date[0]).'"/> <input id="event_end_time" class="format-time" type="text" name="event_end_time" value="'.esc_attr(isset($event_end_date[1]) ? substr($event_end_date[1], 0, 5) : '').'" '.($event_all_day ? 'style="display: none;"' : '').'/>
		</div>
		<div>
			<label for="event_all_day">'.__('All-day event?', 'cdash-events').'</label> <input id="event_all_day" type="checkbox" name="event_all_day" '.checked($event_all_day, true, false).'/>
		</div>';

		// only show recurrences if event is not already part of a recurring series
		$children = get_children('post_parent='.$post->ID.'&post_type=event');
		if( $post->post_parent || $children ) {

			$message = __( 'Are you sure you want to delete all of these events?', 'cdash-events' );
			$url = $this->get_action_url( $_GET['post'] );

			$html .='
			<div class="my_meta_control clearfix event-calendar">
				<h3>' .  __( 'This event is part of a recurring series.', 'cdash-events' ) . '</h3>
				<div class="update-all half">
					<p><strong>' . __( 'Save your changes first!', 'cdash-events' ) . '</strong></p>
					<p><a class="button" id="subsequent" href="#">' . __( 'Update All Events in This Series', 'cdash-events' ) . '</a></p>
					<p>'  .  __( 'This will only apply to events with a scheduled date later than this one.  All information, except the date, will be updated.', 'cdash-events') . '</p>
					<div id="results"></div>
				</div>

				<div class="delete-all half">
					<p><a class="delete button" href="#" onclick="if(confirm(\'' . $message . '\')) { window.location=\'' . $url . '\'; }">' . __( 'Delete All Events In This Series', 'cdash-events' ) . '</a></p>
					<p>' . __( 'This will delete events of all dates.', 'cdash-events' ). '</p>
				</div>
			</div>';



		} else {
			$html .= '
			<div class="date-time-row">
				<label for="event_recurrence">'.__('Recurrence', 'cdash-events').'</label> <select id="event_recurrence" name="event_recurrence[type]" class="">';

			foreach($this->recurrences as $id => $recurrence)
			{
				if($id === 'weekly')
				{
					if($event_recurrence['type'] !== 'weekly')
						$check_options = array($today);
					else
						$check_options = $event_recurrence['weekly_days'];

					global $weekday;

					$weekdays = $weekday;

					if($this->options['general']['first_weekday'] === 1)
					{
						$weekdays[7] = $weekday[0];
						unset($weekdays[0]);
					}
					else
						$weekdays = array_combine(range(1, 7), array_values($weekdays));
					
					$options['weekly'] .= '<fieldset>';
					foreach($weekdays as $day_id => $day)
					{
						$options['weekly'] .= '<input id="event_recurrence_weekday_'.$day_id.'" type="checkbox" name="event_recurrence[weekly][weekly_days]['.$day_id.']" value="'.$day_id.'" '.checked(in_array($day_id, $check_options), true, false).'/><label for="event_recurrence_weekday_'.$day_id.'">'.esc_html($day).'</label>';
					}
					$options['weekly'] .= '</fieldset>';
				}
				elseif($id === 'monthly')
				{
					if($event_recurrence['type'] !== 'monthly')
						$day_type = 1;
					else
						$day_type = (int)$event_recurrence['monthly_day_type'];

					$options['monthly'] = '<input id="event_recurrence_day_month" type="radio" name="event_recurrence[monthly][monthly_day_type]" value="1"/ '.checked($day_type, 1, false).'><label for="event_recurrence_day_month">'.__('day of the month', 'cdash-events').'</label><br /><input id="event_recurrence_day_week" type="radio" name="event_recurrence[monthly][monthly_day_type]" value="2" '.checked($day_type, 2, false).'/><label for="event_recurrence_day_week">'.__('day of the week', 'cdash-events').'</label>';
				}

				$html .= '
					<option value="'.esc_attr($id).'" '.selected($id, $event_recurrence['type'], false).'>'.esc_html($recurrence).'</option>';
			}

			$template = '
			'.__('Repeat every', 'cdash-events').' <input type="text" size="2" maxlength="4" name="event_recurrence[repeat]" value="%1$d"/> <span class="occurrence">%2$s</span> '.__('until', 'cdash-events').' <input class="event_recurrence_until" type="text" name="event_recurrence[until]" size="10" maxlength="10" value="%3$s"/>';

			if($event_recurrence['type'] === 'once' || $event_recurrence['type'] === 'daily' || $event_recurrence['type'] === 'custom')
				$repeat = _n('day', 'days', $event_recurrence['repeat'], 'cdash-events');
			elseif($event_recurrence['type'] === 'weekly')
				$repeat = _n('week', 'weeks', $event_recurrence['repeat'], 'cdash-events');
			elseif($event_recurrence['type'] === 'monthly')
				$repeat = _n('month', 'months', $event_recurrence['repeat'], 'cdash-events');
			else
				$repeat = _n('year', 'years', $event_recurrence['repeat'], 'cdash-events');

			$html .= '
				</select>
				<div id="event_recurrence_types" class="date-time-row"'.(in_array($event_recurrence['type'], array('custom', 'once'), true) ? ' style="display: none;"' : '').'>
					<div class="start">
						'.sprintf(
							$template,
							$event_recurrence['repeat'],
							$repeat,
							$event_recurrence['until']
						).'
					</div>
					<div class="weekly"'.($event_recurrence['type'] === 'weekly' ? '' : ' style="display: none;"').'>
						'.$options['weekly'].'
					</div>
					<div class="monthly"'.($event_recurrence['type'] === 'monthly' ? '' : ' style="display: none;"').'>
						'.$options['monthly'].'
					</div>
				</div>
				<div id="event_custom_occurrences" class="date-time-row"'.($event_recurrence['type'] === 'custom' ? '' : ' style="display: none;"').'>
					<div id="event-custom-template" style="display: none;">
						<div class="event-custom" style="display: none;">
							<label for="event_separate_end_date____ID___">'.__('Separate end date', 'cdash-events').'</label><input class="event_separate" id="event_separate_end_date____ID___" type="checkbox" name="___EVENT_CUSTOM_DATE___[separate_end_date][___ID___]"/>
							<div class="start">
								<label for="event_start_date____ID___">'.__('Start date/time', 'cdash-events').':</label> <input id="event_start_date____ID___" type="text" name="___EVENT_CUSTOM_DATE___[dates][___ID___][start][date]" value="" class="event_custom_date"/> <input id="event_start_time____ID___" type="text" name="___EVENT_CUSTOM_DATE___[dates][___ID___][start][time]" value="" class="event_custom_time"/>
								<a class="delete-custom-event button button-secondary" href="#">'.__('Delete', 'cdash-events').'</a>
							</div>
							<div class="end" style="display: none">
								<label for="event_end_date____ID___">'.__('End date/time', 'cdash-events').':</label> <input id="event_end_date____ID___" type="text" name="___EVENT_CUSTOM_DATE___[dates][___ID___][end][date]" value="" class="event_custom_date"/> <input id="event_end_time____ID___" type="text" name="___EVENT_CUSTOM_DATE___[dates][___ID___][end][time]" value="" class="event_custom_time"/>
							</div>
						</div>
					</div>
					<div>
						<a href="#" id="add-custom-event" class="button button-primary">'.__('Add new occurrence', 'cdash-events').'</a>
					</div>';

			if($event_recurrence['type'] === 'custom')
			{
				$occurrences = get_post_meta($post->ID, '_event_occurrence_date', false);

				if(!empty($occurrences))
				{
					foreach($occurrences as $id => $occurrence)
					{
						if($id === 0)
							continue;

						$dates = explode('|', $occurrence);

						if($event_all_day)
						{
							$start['date'] = date('Y-m-d', strtotime($dates[0]));
							$start['time'] = '';
							$end['date'] = date('Y-m-d', strtotime($dates[1]));
							$end['time'] = '';
						}
						else
						{
							$start['date'] = date('Y-m-d', strtotime($dates[0]));
							$start['time'] = date('H:i', strtotime($dates[0]));
							$end['date'] = date('Y-m-d', strtotime($dates[1]));
							$end['time'] = date('H:i', strtotime($dates[1]));
						}

						$html .= '
						<div class="event-custom">
							<label for="event_separate_end_date_'.$id.'">'.__('Separate end date', 'cdash-events').'</label><input class="event_separate" id="event_separate_end_date_'.$id.'" type="checkbox" name="event_recurrence[custom][separate_end_date]['.$id.']" '.checked($event_recurrence['separate_end_date'][$id - 1], true, false).'/>
							<div class="start">
								<label for="event_start_date_'.$id.'">'.__('Start date/time', 'cdash-events').':</label> <input id="event_start_date_'.$id.'" type="text" name="event_recurrence[custom][dates]['.$id.'][start][date]" value="'.$start['date'].'" class="event_custom_date"/> <input id="event_start_time_'.$id.'" type="text" name="event_recurrence[custom][dates]['.$id.'][start][time]" value="'.$start['time'].'" class="event_custom_time"/>
								<a class="delete-custom-event button button-secondary" href="#">'.__('Delete', 'cdash-events').'</a>
							</div>
							<div class="end"'.($event_recurrence['separate_end_date'][$id - 1] ? '' : ' style="display: none"').'>
								<label for="event_end_date_'.$id.'">'.__('End date/time', 'cdash-events').':</label> <input id="event_end_date_'.$id.'" type="text" name="event_recurrence[custom][dates]['.$id.'][end][date]" value="'.$end['date'].'" class="event_custom_date"/> <input id="event_end_time_'.$id.'" type="text" name="event_recurrence[custom][dates]['.$id.'][end][time]" value="'.$end['time'].'" class="event_custom_time"/>
							</div>
						</div>';
					}
				}
			}
			else
			{
				$html .= '
					<div class="event-custom">
						<label for="event_separate_end_date_1">'.__('Separate end date', 'cdash-events').'</label><input class="event_separate" id="event_separate_end_date_1" type="checkbox" name="event_recurrence[custom][separate_end_date][1]"/>
						<div class="start">
							<label for="event_start_date_1">'.__('Start date/time', 'cdash-events').':</label> <input id="event_start_date_1" type="text" name="event_recurrence[custom][dates][1][start][date]" value="" class="event_custom_date"/> <input id="event_start_time_0" type="text" name="event_recurrence[custom][dates][1][start][time]" value="" class="event_custom_time"/>
							<a class="delete-custom-event button button-secondary" href="#">'.__('Delete', 'cdash-events').'</a>
						</div>
						<div class="end" style="display: none">
							<label for="event_end_date_1">'.__('End date/time', 'cdash-events').':</label> <input id="event_end_date_1" type="text" name="event_recurrence[custom][dates][1][end][date]" value="" class="event_custom_date"/> <input id="event_end_time_0" type="text" name="event_recurrence[custom][dates][1][end][time]" value="" class="event_custom_time"/>
						</div>
					</div>';
			}

			$html .= '
				</div>
			</div>';
		}
		
		do_action('cde_before_metabox_event_datetime');

		echo $html;

		do_action('cde_after_metabox_event_datetime');
	}


	/**
	 * Event tickets metabox callback
	*/
	public function event_tickets_cb($post)
	{
		wp_nonce_field('cdash_events_save_event_tickets', 'event_nonce_tickets');

		$tickets = get_post_meta($post->ID, '_event_tickets', true);
		$free_event = (($free = get_post_meta($post->ID, '_event_free', true)) === '' ? '1' : $free);
		$html_t = '';
		$symbol = cde_get_currency_symbol();

		$html = '
		<p>
			<label for="event_free">'.__('Is this a free event?', 'cdash-events').'</label>
			<input id="event_free" type="checkbox" name="event_free" '.checked($free_event, '1', false).' /> 
		</p>
		<div id="event_cost_and_tickets"'.($free_event === '1' ? ' style="display: none;"' : '').'>
			<div>
				<a href="#" id="event_add_ticket" class="button">'.__('Add new ticket', 'cdash-events').'</a>
			</div>';

		if(!empty($tickets) && is_array($tickets))
		{
			foreach($tickets as $id => $ticket)
			{
				$html_t .= '
				<p rel="'.$id.'">';

				foreach($this->tickets_fields as $key => $field)
				{
					$html_t .= '
					<label for="event_tickets['.$id.']['.$key.']">'.$field.':</label> <input type="text" id="event_tickets['.$id.']['.$key.']" name="event_tickets['.$id.']['.$key.']" value="'.esc_attr(isset($ticket[$key]) ? $ticket[$key] : '').'" />'.($key === 'price' ? $symbol : '');
				}

				$html_t .= '
					<a href="#" class="event_ticket_delete button button-secondary">'.__('Delete', 'cdash-events').'</a>
				</p>';
			}
		}
		else
		{
			$html_t .= '
				<p rel="0">';

			foreach($this->tickets_fields as $key => $field)
			{
				$html_t .= '
					<label for="event_tickets[0]['.$key.']">'.$field.':</label> <input type="text" id="event_tickets[0]['.$key.']" name="event_tickets[0]['.$key.']" value="" />'.($key === 'price' ? $symbol : '');
			}

			$html_t .= '
					<a href="#" class="event_ticket_delete button button-secondary">'.__('Delete', 'cdash-events').'</a>
				</p>';
		}

		$html .= '
			<div id="event_tickets">
			'.$html_t.'
			</div>';

		$sales_location = get_post_meta($post->ID, '_sales_location', true);
	    $display = "block";
	    if( isset( $sales_location ) && "sell_here" == $sales_location ) {
	        $display = "none";
	    }

		$more_html = '
			<div id="tickets_url" style="display: ' . $display . '">
				<label for="event_tickets_url">'.__('Buy tickets URL (external site)', 'cdash-events').':</label><br />' 
                .__('Name of the site', 'cdash-events').': <input type="text" id="event_tickets_url_name" class="regular-text" name="event_tickets_url_name" value="'.get_post_meta($post->ID, '_event_tickets_url_name', true).'" /><br />'
                .__('Url of the site', 'cdash-events').': <input id="event_tickets_url" class="regular-text" type="text" name="event_tickets_url" value="'.esc_url(get_post_meta($post->ID, '_event_tickets_url', true)).'" />                
			</div>
		</div>';
		
		global $post;
		do_action('cde_before_metabox_event_tickets');

		echo $html;

		do_action('cde_after_metabox_event_tickets');

		echo $more_html;
		
	}


	/**
	 * Event options metabox callback
	*/
	public function event_options_cb($post)
	{
		// security nonce
		wp_nonce_field('cdash_events_save_event_options', 'event_nonce_options');

		$options = array(
			'google_map' => __('Display Google Map', 'cdash-events'),
			'display_location_details' => __('Display Location Details', 'cdash-events'),
			'price_tickets_info' => __('Display Tickets Info', 'cdash-events')
		);

		$options = apply_filters('cde_metabox_event_display_options', $options, $post);
		$values = apply_filters('cde_metabox_event_display_values', get_post_meta($post->ID, '_event_display_options', true), $post);
		
		// display options
		do_action('cde_before_metabox_event_display_options', $post);

		foreach($options as $key => $name)
		{
			?>
			<label for="event_display_option_<?php echo $key; ?>">
				<input id="event_display_option_<?php echo $key; ?>" type="checkbox" name="event_display_options[<?php echo $key; ?>]" <?php checked((isset($values[$key]) && $values[$key] !== '' ? $values[$key] : '1'), (isset($this->options['general']['default_event_options'][$key]) ? (bool)$this->options['general']['default_event_options'][$key] : '0')); ?> /><?php echo $name; ?>
			</label><br />
			<?php
		}

		do_action('cde_after_metabox_event_display_options', $post);
	}


	/**
	 * Save event metaboxes data
	*/
	public function save_event($post_ID)
	{

		// break if doing autosave
		if(defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
			return $post_ID;

		// don't do this twice
		if( wp_is_post_revision( $post_ID) || wp_is_post_autosave( $post_ID ) )
			return $post_ID;

		// break if current user can't edit events
		if(!current_user_can('edit_posts', $post_ID))
			return $post_ID;

		// event display options validation				
		if(isset($_POST['event_nonce_options']) && wp_verify_nonce($_POST['event_nonce_options'], 'cdash_events_save_event_options'))
		{
			$event_display_options = array(
				'google_map' => __('Display Google Map', 'cdash-events'),
				'display_location_details' => __('Display Location Details', 'cdash-events'),
				'price_tickets_info' => __('Display Tickets Info', 'cdash-events')
			);

			$event_display_options = apply_filters('cde_metabox_event_display_options', $event_display_options, $post_ID);
	
			if(is_array($event_display_options) && !empty($event_display_options))
			{
				$event_display_values = array();
	
				foreach($event_display_options as $key => $name)
				{
					$event_display_values[$key] = isset($_POST['event_display_options'][$key]) ? 1 : 0;
				}
	
				update_post_meta($post_ID, '_event_display_options', $event_display_values);
			}
			else
				update_post_meta($post_ID, '_event_display_options', array());
		}

		// event tickets validation, if tickets are in use
		if(isset($_POST['event_nonce_tickets']) && wp_verify_nonce($_POST['event_nonce_tickets'], 'cdash_events_save_event_tickets'))
		{
			update_post_meta($post_ID, '_event_free', (isset($_POST['event_free']) ? 1 : 0));

			$tickets = array();

			if(!isset($_POST['event_free']))
			{
				$ticket_url = (isset($_POST['event_tickets_url']) ? $_POST['event_tickets_url'] : '');
                $ticket_url_name = (isset($_POST['event_tickets_url_name']) ? $_POST['event_tickets_url_name'] : '');

				if(isset($_POST['event_tickets']) && is_array($_POST['event_tickets']) && !empty($_POST['event_tickets']))
				{
					foreach($_POST['event_tickets'] as $id => $ticket)
					{
						$tickets_fields = array();
						$empty = 0;

						foreach($this->tickets_fields as $key => $trans)
						{
							$tickets_fields[$key] = sanitize_text_field(isset($ticket[$key]) ? $ticket[$key] : '');
							$empty += (($tickets_fields[$key] !== '') ? 1 : 0);
						}

						if($empty > 0)
							$tickets[$id] = $tickets_fields;
					}

					if(empty($tickets))
					{
						$ticket_url = '';

						update_post_meta($post_ID, '_event_free', 1);
					}

					update_post_meta($post_ID, '_event_tickets', $tickets);
				}
				else
				{
					$ticket_url = '';

					update_post_meta($post_ID, '_event_tickets', array());
					update_post_meta($post_ID, '_event_free', 1);
				}

				update_post_meta($post_ID, '_event_tickets_url', esc_url($ticket_url));
                update_post_meta($post_ID, '_event_tickets_url_name', $ticket_url_name);
			}
			else
			{
				update_post_meta($post_ID, '_event_tickets', $tickets);
				update_post_meta($post_ID, '_event_tickets_url', '');
                update_post_meta($post_ID, '_event_tickets_url_name', '');
			}

			// ticket sales validation

			if( isset( $_POST['sales_location'] ) ) {

				update_post_meta( $post_ID, '_sales_location', $_POST['sales_location'] );

				if( 'sell_here' == $_POST['sales_location'] ) {

					$tix_number = sanitize_text_field( $_POST['tix_number'] );
					update_post_meta( $post_ID, '_tix_number', $tix_number );
					// $msg = "<pre>" . get_post_meta( $post_ID, '_tix_number', true) . "</pre>";
					// wp_die($msg);

					if($cde_helper->is_valid_date($_POST['tix_sales_open']) && $cde_helper->is_valid_time($_POST['tix_sales_start_time']))
						{
							$start_date_ok = true;
							$tix_sales_open = date('Y-m-d H:i:s', strtotime($_POST['tix_sales_open'].' '.$_POST['tix_sales_start_time']));
							update_post_meta($post_ID, '_tix_sales_open', $tix_sales_open);
						}
						else
						{
							update_post_meta($post_ID, '_tix_sales_open', $current_datetime);
							$tix_sales_open = $current_datetime;
						}
			
						if($cde_helper->is_valid_date($_POST['tix_sales_close']) && $cde_helper->is_valid_time($_POST['tix_sales_close_time']))
						{
							if($start_date_ok)
							{
								if($cde_helper->is_after_date($_POST['tix_sales_close'].' '.$_POST['tix_sales_close_time'], $_POST['tix_sales_open'].' '.$_POST['event_start_time']))
								{
									$tix_sales_close = date('Y-m-d H:i:s', strtotime($_POST['tix_sales_close'].' '.$_POST['tix_sales_close_time']));
									update_post_meta($post_ID, '_tix_sales_close', $tix_sales_close);
								}
								else
								{
									$tix_sales_close = $tix_sales_open;
									update_post_meta($post_ID, '_tix_sales_close', $tix_sales_close);
								}
							}
							else
							{
								$tix_sales_close = date('Y-m-d H:i:s', strtotime($_POST['tix_sales_close'].' '.$_POST['tix_sales_close_time']));
								update_post_meta($post_ID, '_tix_sales_close', $tix_sales_close);
							}
						}
						else
						{
							$tix_sales_close = $tix_sales_open;
							update_post_meta($post_ID, '_tix_sales_close', $tix_sales_close);
						}

					}

				}

		}

		// event date & time validation
		if(isset($_POST['event_nonce_datetime']) && wp_verify_nonce($_POST['event_nonce_datetime'], 'cdash_events_save_event_datetime'))
		{
			// event date and time section
			$cde_helper = new Cdash_Events_Helper();
			$event_all_day = isset($_POST['event_all_day']) ? 1 : 0;
			$start_date_ok = false;
	
			update_post_meta($post_ID, '_event_all_day', $event_all_day);
			$event_start_date = $event_end_date = $current_datetime = current_time('mysql', false);
			$current_date = date('Y-m-d', current_time('timestamp', false));
	
			// is it all day long event?
			if($event_all_day === 1)
			{
				if($cde_helper->is_valid_date($_POST['event_start_date']))
				{
					$start_date_ok = true;
					update_post_meta($post_ID, '_event_start_date', $_POST['event_start_date'].' 00:00:00');
					$event_start_date = $_POST['event_start_date'].' 00:00:00';
				}
				else
				{
					update_post_meta($post_ID, '_event_start_date', $current_datetime);
					$event_start_date = $current_datetime;
				}
	
				if($cde_helper->is_valid_date($_POST['event_end_date']))
				{
					if($start_date_ok)
					{
						if($cde_helper->is_after_date($_POST['event_end_date'], $_POST['event_start_date']))
						{
							update_post_meta($post_ID, '_event_end_date', $_POST['event_end_date'].' 00:00:00');
							$event_end_date = $_POST['event_end_date'].' 00:00:00';
						}
						else
						{
							$event_end_date = $event_start_date;
							update_post_meta($post_ID, '_event_end_date', $event_end_date);
						}
					}
					else
					{
						update_post_meta($post_ID, '_event_end_date', $_POST['event_end_date'].' 00:00:00');
						$event_end_date = $_POST['event_end_date'].' 00:00:00';
					}
				}
				else
				{
					$event_end_date = $event_start_date;
					update_post_meta($post_ID, '_event_end_date', $event_end_date);
				}
			}
			else
			{
				if($cde_helper->is_valid_date($_POST['event_start_date']) && $cde_helper->is_valid_time($_POST['event_start_time']))
				{
					$start_date_ok = true;
					$event_start_date = date('Y-m-d H:i:s', strtotime($_POST['event_start_date'].' '.$_POST['event_start_time']));
					update_post_meta($post_ID, '_event_start_date', $event_start_date);
				}
				else
				{
					update_post_meta($post_ID, '_event_start_date', $current_datetime);
					$event_start_date = $current_datetime;
				}
	
				if($cde_helper->is_valid_date($_POST['event_end_date']) && $cde_helper->is_valid_time($_POST['event_end_time']))
				{
					if($start_date_ok)
					{
						if($cde_helper->is_after_date($_POST['event_end_date'].' '.$_POST['event_end_time'], $_POST['event_start_date'].' '.$_POST['event_start_time']))
						{
							$event_end_date = date('Y-m-d H:i:s', strtotime($_POST['event_end_date'].' '.$_POST['event_end_time']));
							update_post_meta($post_ID, '_event_end_date', $event_end_date);
						}
						else
						{
							$event_end_date = $event_start_date;
							update_post_meta($post_ID, '_event_end_date', $event_end_date);
						}
					}
					else
					{
						$event_end_date = date('Y-m-d H:i:s', strtotime($_POST['event_end_date'].' '.$_POST['event_end_time']));
						update_post_meta($post_ID, '_event_end_date', $event_end_date);
					}
				}
				else
				{
					$event_end_date = $event_start_date;
					update_post_meta($post_ID, '_event_end_date', $event_end_date);
				}
			}

			// removes all previous occurrences
			delete_post_meta($post_ID, '_event_occurrence_date');
			delete_post_meta($post_ID, '_event_occurrence_last_date');
	
			// adds first occurrence even for one time events
			update_post_meta($post_ID, '_event_occurrence_date', $event_start_date.'|'.$event_end_date);
	
			$recurrence = $_POST['event_recurrence'];

			if(isset($this->recurrences[$recurrence['type']]))
			{
				$recurrence['until'] = date('Y-m-d', strtotime($recurrence['until']));
				$today = (int)date('N', strtotime($event_start_date));
	
				if($recurrence['type'] === 'once')
				{
					$event_recurrence = array(
						'type' => 'once',
						'repeat' => 1,
						'until' => $current_date,
						'weekly_days' => array($today),
						'monthly_day_type' => 1,
						'separate_end_date' => array()
					);
	
					// adds last occurrence (same as first)
					update_post_meta($post_ID, '_event_occurrence_date', $event_start_date.'|'.$event_end_date);
					update_post_meta($post_ID, '_event_occurrence_last_date', $event_start_date.'|'.$event_end_date);
				}
				elseif($recurrence['type'] === 'custom')
				{
					$event_recurrence = array(
						'type' => 'custom',
						'repeat' => 1,
						'until' => $current_date,
						'weekly_days' => array($today),
						'monthly_day_type' => 1
					);
	
					if(!empty($recurrence['custom']['separate_end_date']))
						$separates = $recurrence['custom']['separate_end_date'];
					else
						$separates = array();
	
					// adds custom dates
					$event_recurrence['separate_end_date'] = $this->add_custom_dates($post_ID, $recurrence['custom']['dates'], $event_all_day, $separates, $event_start_date, $event_end_date);
				}
				else
				{
					$weekly_days = array();
					$monthly_day_type = 1;
	
					if($recurrence['type'] === 'weekly')
					{
						if(isset($recurrence['weekly']['weekly_days']))
						{
							foreach($recurrence['weekly']['weekly_days'] as $week_id => $weekday)
							{
								$id = (int)$week_id;
	
								if($id >= 1 && $id <= 7)
									$weekly_days[] = $id;
							}
	
							if(empty($weekly_days))
								$weekly_days = array($today);
						}
						else
							$weekly_days = array($today);
					}
					elseif($recurrence['type'] === 'monthly')
					{
						$weekly_days = array($today);
	
						if(isset($recurrence['monthly']['monthly_day_type']))
						{
							$id = (int)$recurrence['monthly']['monthly_day_type'];
	
							$monthly_day_type = ($id === 2 ? 2 : 1);
						}
					}
	
					$event_recurrence = array(
						'type' => $recurrence['type'],
						'repeat' => (($repeat = (int)$recurrence['repeat']) > 0 ? $repeat : 1),
						'until' => $recurrence['until'],
						'weekly_days' => $weekly_days,
						'monthly_day_type' => $monthly_day_type,
						'separate_end_date' => array()
					);

					// creates occurrences
					$this->create_recurrences($post_ID, $event_start_date, $event_end_date, $recurrence['type'], $recurrence['repeat'], $recurrence['until'], $weekly_days, $monthly_day_type);
				}
			}
	
			update_post_meta($post_ID, '_event_recurrence', $event_recurrence);

		}
		

		

	}

	/**
	 * Add custom recurrence dates
	*/
	private function add_custom_dates($post_id, $dates, $all_day, $separate_end_date, $start, $end)
	{
		$custom_dates = $separates = array();
		$diff = strtotime($end) - strtotime($start);
		$cde_helper = new Cdash_Events_Helper();

		// is it all day long event?
		if($all_day === 1)
		{
			foreach($dates as $id => $date)
			{
				if(isset($separate_end_date[$id]))
				{
					if($cde_helper->is_valid_date($date['start']['date']) && $cde_helper->is_valid_date($date['end']['date']) && $cde_helper->is_after_date($date['end']['date'], $date['start']['date'], false))
					{
						$separates[] = true;
						$custom_dates[] = array(
							'start' => $date['start']['date'].' 00:00:00',
							'end' => $date['end']['date'].' 00:00:00'
						);
					}
				}
				else
				{
					if($cde_helper->is_valid_date($date['start']['date']))
					{
						$separates[] = false;
						$custom_dates[] = array(
							'start' => $date['start']['date'].' 00:00:00',
							'end' => date('Y-m-d', strtotime($date['start']['date']) + $diff).' 00:00:00'
						);
					}
				}
			}
		}
		else
		{
			foreach($dates as $id => $date)
			{
				if(isset($separate_end_date[$id]))
				{
					if($cde_helper->is_valid_date($date['start']['date']) && $cde_helper->is_valid_date($date['end']['date']) && $cde_helper->is_valid_time($date['start']['time']) && $cde_helper->is_valid_time($date['end']['time']) && $cde_helper->is_after_date($date['end']['date'].' '.$date['end']['time'], $date['start']['date'].' '.$date['start']['time'], false))
					{
						$separates[] = true;
						$custom_dates[] = array(
							'start' => date('Y-m-d H:i:s', strtotime($date['start']['date'].' '.$date['start']['time'])),
							'end' => date('Y-m-d H:i:s', strtotime($date['end']['date'].' '.$date['end']['time']))
						);
					}
				}
				else
				{
					if($cde_helper->is_valid_date($date['start']['date']) && $cde_helper->is_valid_time($date['start']['time']))
					{
						$time = strtotime($date['start']['date'].' '.$date['start']['time']);
						$separates[] = false;
						$custom_dates[] = array(
							'start' => date('Y-m-d H:i:s', $time),
							'end' => date('Y-m-d H:i:s', $time + $diff)
						);
					}
				}
			}
		}

		if(!empty($custom_dates))
		{
			global $wpdb;

			$query = array();

			foreach($custom_dates as $date)
			{
				$query[] = "(".$post_id.", '_event_occurrence_date', '".$date['start']."|".$date['end']."')";
			}

			if(!empty($query))
				$wpdb->query('INSERT INTO '.$wpdb->postmeta.' (post_id, meta_key, meta_value) VALUES '.implode(', ', $query));

			array_multisort($custom_dates);

			// gets last occurrence
			$last = end($custom_dates);

			// adds last occurrence
			add_post_meta($post_id, '_event_occurrence_last_date', $last['start'].'|'.$last['end']);
		}
		else
		{
			// adds last occurrence (same as first)
			add_post_meta($post_id, '_event_occurrence_last_date', $start.'|'.$end);
		}

		return $separates;
	}

	/**
	 * Create custom recurrence dates
	*/
	private function create_recurrences($post_id, $start, $end, $type, $repeat, $until, $weekly_days, $monthly_day_type)
	{
		$cde_helper = new Cdash_Events_Helper();

		if($cde_helper->is_after_date($start, $end) || $cde_helper->is_after_date($start, $until))
			return;

		$format = 'Y-m-d H:i:s';
		$occurrences = array();
		$diff = strtotime($end) - strtotime($start);
		$finish = strtotime($until);

		if($type === 'daily')
		{
			$repeat *= 86400;
			$current = strtotime($start);

			while($current <= $finish)
			{
				$occurrences[] = array('start' => date($format, $current), 'end' => date($format, $current + $diff));

				// creates new current date
				$current += $repeat;
			}
		}
		elseif($type === 'weekly')
		{
			$current = $start_date = strtotime($start);
			$weekdays = array();
			$repeat *= 7;
			$i = $counter = 0;
			$day = date('N', $current);

			foreach($weekly_days as $weekday)
			{
				$weekdays[] = $weekday - $day;
			}

			$number_of_days = count($weekdays);

			while($current <= $finish)
			{
				if(($more_days = ($weekdays[$i++] + $repeat * $counter)) >= 0)
				{
					// creates new current date
					$current = strtotime('+'.$more_days.' days', $start_date);

					if($current <= $finish)
						$occurrences[] = array('start' => date($format, $current), 'end' => date($format, $current + $diff));
				}

				if($i === $number_of_days)
				{
					$counter++;
					$i = 0;
				}
			}
		}
		elseif($type === 'monthly')
		{
			$current = strtotime($start);
			$start_date = date_parse($start);

			// is it day of week?
			if($monthly_day_type === 2)
			{
				// 1-7
				$day_of_week = date('N', $current);

				// 1-31 / 7 rounded down
				$which = (int)floor(date('j', $current) / 7);

				// time
				$diff_time = $start_date['second'] + $start_date['minute'] * 60 + $start_date['hour'] * 3600;
			}
			else
				$diff_time = 0;

			while($current <= $finish)
			{
				$occurrences[] = array('start' => date($format, $current + $diff_time), 'end' => date($format, $current + $diff + $diff_time));

				// current date
				$date = date_parse(date('Y-m-d', $current));

				// creates new current date
				if($start_date['day'] > 28)
				{
					$values = date('Y-m-t', strtotime('+'.$repeat.' months', strtotime($date['year'].'-'.$date['month'].'-01')));
					$values = explode('-', $values);

					if($values[2] < $date['day'])
						$current = strtotime($values[0].'-'.$values[1].'-'.$values[2]);
					else
						$current = strtotime($values[0].'-'.$values[1].'-'.$start_date['day']);
				}
				else
					$current = strtotime('+'.$repeat.' months', $current);

				if($monthly_day_type === 2)
				{
					// due to PHP 5.2 bugs lets do some craziness
					$year = date('Y', $current);
					$month = date('m', $current);
					$day_of_month = date('N', strtotime($year.'-'.$month.'-01'));

					if($day_of_month <= $day_of_week)
						$number = $day_of_week - $day_of_month + 1;
					else
						$number = $day_of_week - $day_of_month + 8;

					$number += (7 * $which);

					// is it valid date?
					while(!checkdate((int)$month, $number, $year))
					{
						$number -= 7;
					}

					$current = strtotime($year.'-'.$month.'-'.str_pad($number, 2, '0', STR_PAD_LEFT));
				}
			}
		}
		elseif($type === 'yearly')
		{
			$current = strtotime($start);

			while($current <= $finish)
			{
				$occurrences[] = array('start' => date($format, $current), 'end' => date($format, $current + $diff));

				// creates new current date
				$current = strtotime('+1 year', $current);
			}
		}

		$i = 0;

		if(!empty($occurrences))
		{
			// avoid infinite loops
			remove_action('save_post', array(&$this, 'save_event'), 10, 2);

			foreach( $occurrences as $occurrence ) {

				if( "0" == $i ) { // don't make a copy of the first event
					$i++;
					continue;
				}

				$original_event = get_post( $post_id );

				$args = array(
					'comment_status' => $original_event->comment_status,
					'ping_status'    => $original_event->ping_status,
					'post_author'    => $original_event->post_author,
					'post_content'   => $original_event->post_content,
					'post_excerpt'   => $original_event->post_excerpt,
					'post_name'      => $original_event->post_name,
					'post_parent'    => $post_id,
					'post_password'  => $original_event->post_password,
					'post_status'    => $original_event->post_status,
					'post_title'     => $original_event->post_title,
					'post_type'      => $original_event->post_type,
				);

				$recurring_event = wp_insert_post( $args );

				// copy all of the categories, tags, and locations
				$taxonomies = get_object_taxonomies($original_event->post_type); // returns array of taxonomy names for post type, ex array("category", "post_tag");
				foreach ($taxonomies as $taxonomy) {
					$post_terms = wp_get_object_terms($post_id, $taxonomy, array('fields' => 'slugs'));
					wp_set_object_terms($recurring_event, $post_terms, $taxonomy, false);
				}

				// copy all of the post meta data 
				$meta = get_post_meta($post_id);
 
		        foreach ( $meta as $key => $values) {
		            foreach ($values as $value) {
		            	if(@unserialize($value) !== false) {
		            		$value = unserialize($value);
		            	}
		                add_post_meta( $recurring_event, $key, $value );
		            }
		        }

		        // now replace the post meta data with this occurrence dates and times
		        update_post_meta( $recurring_event, '_event_start_date', $occurrence['start'] );
		        update_post_meta( $recurring_event, '_event_end_date', $occurrence['end'] );

		        $i++;
			}

			add_action('save_post', array(&$this, 'save_event'), 10, 2);
		}
	}

	/**
	 * Delete all events in a series
	*/

	private function get_action_url( $post_id ) {
		return wp_nonce_url( add_query_arg( array( 'action' => 'cdash_events_completely_delete', 'post' => $post_id ), admin_url( 'admin.php' ) ), 'cdash_events_delete_series', 'cdash_events_delete_nonce' );
	}

	public function event_row_actions( $actions, $post ) {
		$children = get_children('post_parent='.$post->ID.'&post_type=event');
		if ( 'trash' != $post->post_status && $post->post_type == 'event' && ( $post->post_parent || $children ) ) {
			$message = __( 'Are you sure you want to delete all of these events?', 'cdash-events' );
			$url = $this->get_action_url( $post->ID );
			$actions['cdash-events'] = '<a class="submitcd delete" href="#" onclick="if(confirm(\'' . $message . '\')) { window.location=\'' . $url . '\'; }">' . __( 'Delete All Events In Series', 'cdash-events' ) . '</a>';
		}

		return $actions;
	}

	function completely_delete() {
		if (!isset($_GET['cdash_events_delete_nonce']) || !wp_verify_nonce($_GET['cdash_events_delete_nonce'], 'cdash_events_delete_series')) {
			// do nothing
		} else {
			if(isset($_REQUEST['post'])) {
				$post_id = $_REQUEST['post'];
				$poststodelete = array();
				$i= 0;
				// get the children of this post, if there are any
				$children = get_posts('post_type=event&posts_per_page=-1&post_parent=' . $post_id);
				foreach($children as $child) {
					$poststodelete[] = $child->ID;
					$i++;
				}

				// get the parent of this post, if there is one
				$parent = wp_get_post_parent_id( $post_id );
				if(isset($parent) && $parent != '') {
					$poststodelete[] = $parent;
					$i++;
					// get the other children of this post's parent
					$siblings = get_posts('post_type=event&posts_per_page=-1&post_parent=' . $parent);
					foreach($siblings as $sibling) {
						$poststodelete[] = $sibling->ID;
						$i++;
					}
				}

				// delete all of the children and parents
				foreach($poststodelete as $postid) {
					wp_trash_post($postid);
				}
				wp_trash_post($post_id);  // delete the original post
				wp_redirect( admin_url( '/edit.php?post_type=event' ) );
			}

		}

	}

	/**
	 * Apply changes to subsequent events in a series
	*/

	public function apply_changes_to_series() {
		if( !isset( $_POST['eventid'] ) ) {
			$results = __( 'Error.  No events were found.', 'cdash-events' );
			die( $results );
		}

		$post_id = $_POST['eventid'];

		$original_event = get_post( $post_id );

		// gather all of this shift's information
		$title = get_the_title( $post_id );
		$content = get_post_field( 'post_content', $post_id );

		$event_category = wp_get_post_terms( $post_id, 'event-category', array("fields" => "ids") );
		$event_tag = wp_get_post_terms( $post_id, 'event-tag', array("fields" => "ids") );
		$event_location = wp_get_post_terms( $post_id, 'event-location', array("fields" => "ids") );

		$start = get_post_meta( $post_id, '_event_start_date', true );
		$start_time = esc_attr( substr($start, 11, 8) );
		$end = get_post_meta( $post_id, '_event_end_date', true );
		$end_time = esc_attr( substr($end, 11, 8) );
		$display = get_post_meta( $post_id, '_event_display_options', true );
		$all_day = get_post_meta( $post_id, '_event_all_day', true );
		$free = get_post_meta( $post_id, '_event_free', true );
		$tickets = get_post_meta( $post_id, '_event_tickets', true );
		$tickets_url = get_post_meta( $post_id, '_event_tickets_url', true );
        $tickets_url_name = get_post_meta( $post_id, '_event_tickets_url_name', true );


		// find all of the subsequent events
		$parent = wp_get_post_parent_id( $post_id );
		if( 0 === $parent ) {
			// this doesn't have a parent, so we need to get the children
			$args = array(
				'post_type' => 'event',
				'posts_per_page' => '-1',
				'post_parent' => $post_id,
				'meta_key' => '_event_start_date',
				'meta_value' => $start,
				'meta_compare' => '>',
			);
		} else {
			// this is a child post, so we need to get the siblings
			$args = array(
				'post_type' => 'event',
				'posts_per_page' => '-1',
				'post_parent' => $parent,
				'meta_key' => '_event_start_date',
				'meta_value' => $start,
				'meta_compare' => '>',
			);
		}
		
		$subsequent_events = new WP_Query( $args );

		$i = 0;
		
		// update all subsequent events
		if ( $subsequent_events->have_posts() ) :
			while ( $subsequent_events->have_posts() ) : $subsequent_events->the_post();
				$id = get_the_id(); 
				// update basic post information
				$update = array(
					'ID' => $id,
					'post_title' => $title,
					'post_content' => $content,
					'comment_status' => $original_event->comment_status,
					'ping_status'    => $original_event->ping_status,
					'post_author'    => $original_event->post_author,
					'post_content'   => $original_event->post_content,
					'post_excerpt'   => $original_event->post_excerpt,
					'post_name'      => $original_event->post_name,
					'post_password'  => $original_event->post_password,
					'post_status'    => $original_event->post_status,
					);
				wp_update_post( $update );

				// update event meta
				$this_start = get_post_meta( $id, '_event_start_date', true );
				$start_date = esc_attr( substr($this_start, 0, 10) );
				$this_end = get_post_meta( $id, '_event_end_date', true );
				$end_date = esc_attr( substr($this_end, 0, 10) );
			    update_post_meta( $id, '_event_start_date', $start_date . ' ' . $start_time );
			    update_post_meta( $id, '_event_end_date', $end_date . ' ' . $end_time );
			    update_post_meta( $id, '_event_display_options', $display );
				update_post_meta( $id, '_event_all_day', $all_day );
				update_post_meta( $id, '_event_free', $free );
				update_post_meta( $id, '_event_tickets', $tickets );
				update_post_meta( $id, '_event_tickets_url', $tickets_url );
                update_post_meta( $id, '_event_tickets_url_name', $tickets_url_name );

				// update event taxonomies
				wp_set_post_terms( $id, $event_category, 'event-category' );
				wp_set_post_terms( $id, $event_tag, 'event-tag' );
				wp_set_post_terms( $id, $event_location, 'event-location' );

				$i++;
			endwhile;
		endif;
		
		// Reset Post Data
		wp_reset_postdata();

		$results = $i;
		$results .= __( ' events were updated.', 'cdash-events' );

		die( $results );		
	}
	


}
?>